@@ -8,6 +8,7 @@ from openmdao.core.explicitcomponent import ExplicitComponent
 from openmdao.core.implicitcomponent import ImplicitComponent
 from openmdao.core.indepvarcomp import IndepVarComp
 from openmdao.core.analysis_error import AnalysisError
+from openmdao.core.notebook_mode import notebook_mode
 
 # Components
 from openmdao.components.add_subtract_comp import AddSubtractComp
@@ -78,7 +79,8 @@ from openmdao.drivers.genetic_algorithm_driver import SimpleGADriver
 from openmdao.drivers.differential_evolution_driver import DifferentialEvolutionDriver
 from openmdao.drivers.doe_driver import DOEDriver
 from openmdao.drivers.doe_generators import ListGenerator, CSVGenerator, UniformGenerator, \
-    FullFactorialGenerator, PlackettBurmanGenerator, BoxBehnkenGenerator, LatinHypercubeGenerator
+    FullFactorialGenerator, PlackettBurmanGenerator, BoxBehnkenGenerator, LatinHypercubeGenerator, \
+    GeneralizedSubsetGenerator
 
 # System-Building Tools
 from openmdao.utils.options_dictionary import OptionsDictionary
