@@ -1,5 +1,3 @@
-from __future__ import print_function
-
 import unittest
 import os.path
 import importlib
@@ -14,6 +12,7 @@ exclude = [
     'test_suite',
     'tests',
     'test',
+    'assets',  # Script for processing N2 icons.
 ]
 
 directories = []
