@@ -446,11 +446,11 @@ class TestDOEDriver(unittest.TestCase):
             opts = {}
 
         with printoptions(**opts):
-            with self.assertRaises(ValueError) as err:
+            # have to use regex to handle differences in numpy print formats for shape
+            msg = f"Error assigning p1.x = \[ 0.  0.  0.  0.\]: could not broadcast " \
+                  f"input array from shape \(4.*\) into shape \(1.*\)"
+            with self.assertRaisesRegex(ValueError, msg):
                 prob.run_driver()
-            self.assertEqual(str(err.exception),
-                             "Error assigning p1.x = [ 0.  0.  0.  0.]: "
-                             "could not broadcast input array from shape (4) into shape (1)")
 
     def test_uniform(self):
         prob = om.Problem()
@@ -541,8 +541,8 @@ class TestDOEDriver(unittest.TestCase):
         prob = om.Problem()
         model = prob.model
 
-        model.add_subsystem('p1', om.IndepVarComp('x', np.array([0.0, 0.0])), promotes=['*'])
-        model.add_subsystem('p2', om.IndepVarComp('y', np.array([0.0, 0.0])), promotes=['*'])
+        model.set_input_defaults('x', np.array([0.0, 0.0]))
+        model.set_input_defaults('y', np.array([0.0, 0.0]))
         model.add_subsystem('comp', Digits2Num(), promotes=['*'])
 
         model.add_design_var('x', lower=0.0, upper=np.array([1.0, 2.0]))
@@ -570,7 +570,7 @@ class TestDOEDriver(unittest.TestCase):
         prob = om.Problem()
         model = prob.model
 
-        model.add_subsystem('p1', om.IndepVarComp('xy', np.array([0., 0.])), promotes=['*'])
+        model.set_input_defaults('xy', np.array([0., 0.]))
         model.add_subsystem('comp', ParaboloidArray(), promotes=['*'])
 
         model.add_design_var('xy', lower=np.array([-10., -50.]), upper=np.array([10., 50.]))
@@ -607,6 +607,173 @@ class TestDOEDriver(unittest.TestCase):
             self.assertEqual(outputs['xy'][0], expected_case['xy'][0])
             self.assertEqual(outputs['xy'][1], expected_case['xy'][1])
 
+    def test_full_fact_dict_levels(self):
+        # Specifying levels only for one DV, the other is defaulted
+        prob = om.Problem()
+        model = prob.model
+
+        expected = [
+            {'x': np.array([0.]), 'y': np.array([0.]), 'f_xy': np.array([22.00])},
+            {'x': np.array([1.]), 'y': np.array([0.]), 'f_xy': np.array([17.00])},
+
+            {'x': np.array([0.]), 'y': np.array([.5]), 'f_xy': np.array([26.25])},
+            {'x': np.array([1.]), 'y': np.array([.5]), 'f_xy': np.array([21.75])},
+
+            {'x': np.array([0.]), 'y': np.array([1.]), 'f_xy': np.array([31.00])},
+            {'x': np.array([1.]), 'y': np.array([1.]), 'f_xy': np.array([27.00])},
+        ]
+
+        # size = prob.comm.size
+        # rank = prob.comm.rank
+
+        model.add_subsystem('comp', Paraboloid(), promotes=['x', 'y', 'f_xy'])
+        model.set_input_defaults('x', 0.0)
+        model.set_input_defaults('y', 0.0)
+        model.add_design_var('x', lower=0.0, upper=1.0)
+        model.add_design_var('y', lower=0.0, upper=1.0)
+        model.add_objective('f_xy')
+
+        prob.driver = om.DOEDriver(generator=om.FullFactorialGenerator(levels={"y": 3}))
+        prob.driver.add_recorder(om.SqliteRecorder("cases.sql"))
+
+        prob.setup()
+        prob.run_driver()
+        prob.cleanup()
+
+        cr = om.CaseReader("cases.sql")
+        cases = cr.list_cases('driver', out_stream=None)
+
+        self.assertEqual(len(cases), 6)
+
+        for case, expected_case in zip(cases, expected):
+            outputs = cr.get_case(case).outputs
+            self.assertEqual(outputs['x'], expected_case['x'])
+            self.assertEqual(outputs['y'], expected_case['y'])
+            self.assertEqual(outputs['f_xy'], expected_case['f_xy'])
+
+    def test_generalized_subset(self):
+        # All DVs have the same number of levels
+        prob = om.Problem()
+        model = prob.model
+
+        model.set_input_defaults('x', 0.0)
+        model.set_input_defaults('y', 0.0)
+        model.add_subsystem('comp', Paraboloid(), promotes=['x', 'y', 'f_xy'])
+
+        model.add_design_var('x', lower=0.0, upper=1.0)
+        model.add_design_var('y', lower=0.0, upper=1.0)
+        model.add_objective('f_xy')
+
+        prob.driver = om.DOEDriver(generator=om.GeneralizedSubsetGenerator(levels=2, reduction=2))
+        prob.driver.add_recorder(om.SqliteRecorder("cases.sql"))
+
+        prob.setup()
+        prob.run_driver()
+        prob.cleanup()
+
+        expected = [
+            {'x': np.array([0.0]), 'y': np.array([0.0]), 'f_xy': np.array([22.0])},
+            {'x': np.array([1.0]), 'y': np.array([1.0]), 'f_xy': np.array([27.0])},
+        ]
+
+        cr = om.CaseReader("cases.sql")
+        cases = cr.list_cases('driver')
+
+        self.assertEqual(len(cases), 2)
+
+        for case, expected_case in zip(cases, expected):
+            outputs = cr.get_case(case).outputs
+            for name in ('x', 'y', 'f_xy'):
+                self.assertEqual(outputs[name], expected_case[name])
+
+    def test_generalized_subset_dict_levels(self):
+        # Number of variables specified individually for all DVs (scalars).
+        prob = om.Problem()
+        model = prob.model
+
+        model.set_input_defaults('x', 0.0)
+        model.set_input_defaults('y', 0.0)
+        model.add_subsystem('comp', Paraboloid(), promotes=['x', 'y', 'f_xy'])
+
+        model.add_design_var('x', lower=0.0, upper=1.0)
+        model.add_design_var('y', lower=0.0, upper=1.0)
+        model.add_objective('f_xy')
+
+        prob.driver = om.DOEDriver(generator=om.GeneralizedSubsetGenerator(levels={'x': 3, 'y': 6}, reduction=2))
+        prob.driver.add_recorder(om.SqliteRecorder("cases.sql"))
+
+        prob.setup()
+        prob.run_driver()
+        prob.cleanup()
+
+        expected = [
+            {'x': np.array([0.]), 'y': np.array([0.]), 'f_xy': np.array([22.])},
+            {'x': np.array([0.]), 'y': np.array([0.4]), 'f_xy': np.array([25.36])},
+            {'x': np.array([0.]), 'y': np.array([0.8]), 'f_xy': np.array([29.04])},
+            {'x': np.array([1.]), 'y': np.array([0.]), 'f_xy': np.array([17.])},
+            {'x': np.array([1.]), 'y': np.array([0.4]), 'f_xy': np.array([20.76])},
+            {'x': np.array([1.]), 'y': np.array([0.8]), 'f_xy': np.array([24.84])},
+            {'x': np.array([0.5]), 'y': np.array([0.2]), 'f_xy': np.array([20.99])},
+            {'x': np.array([0.5]), 'y': np.array([0.6]), 'f_xy': np.array([24.71])},
+            {'x': np.array([0.5]), 'y': np.array([1.]), 'f_xy': np.array([28.75])},
+        ]
+
+        cr = om.CaseReader("cases.sql")
+        cases = cr.list_cases('driver')
+
+        self.assertEqual(len(cases), 9)
+
+        for case, expected_case in zip(cases, expected):
+            outputs = cr.get_case(case).outputs
+            for name in ('x', 'y', 'f_xy'):
+                self.assertAlmostEqual(outputs[name][0], expected_case[name][0])
+
+    def test_generalized_subset_array(self):
+        # Number of levels specified individually for all DVs (arrays).
+
+        class Digits2Num(om.ExplicitComponent):
+            """
+            Makes from two vectors with 2 elements a 4 digit number.
+            For singe digit integers always gives a unique output number.
+            """
+
+            def setup(self):
+                self.add_input('x', val=np.array([0., 0.]))
+                self.add_input('y', val=np.array([0., 0.]))
+                self.add_output('f', val=0.0)
+
+            def compute(self, inputs, outputs):
+                x = inputs['x']
+                y = inputs['y']
+                outputs['f'] = x[0] * 1000 + x[1] * 100 + y[0] * 10 + y[1]
+
+        prob = om.Problem()
+        model = prob.model
+
+        model.set_input_defaults('x', np.array([0.0, 0.0]))
+        model.set_input_defaults('y', np.array([0.0, 0.0]))
+        model.add_subsystem('comp', Digits2Num(), promotes=['*'])
+
+        model.add_design_var('x', lower=0.0, upper=np.array([1.0, 2.0]))
+        model.add_design_var('y', lower=0.0, upper=np.array([3.0, 4.0]))
+        model.add_objective('f')
+
+        prob.driver = om.DOEDriver(generator=om.GeneralizedSubsetGenerator(levels={'x': 5, 'y': 8}, reduction=14))
+        prob.driver.add_recorder(om.SqliteRecorder("cases.sql"))
+
+        prob.setup()
+        prob.run_driver()
+        prob.cleanup()
+
+        cr = om.CaseReader("cases.sql")
+        cases = cr.list_cases('driver', out_stream=None)
+
+        objs = [int(cr.get_case(case).outputs['f']) for case in cases]
+
+        self.assertEqual(len(objs), 104)  # The number can be verified with standalone pyDOE2
+        # Testing uniqueness. If all elements are unique, it should be the same length as the number of cases
+        self.assertEqual(len(set(objs)), 104)
+
     def test_plackett_burman(self):
         prob = om.Problem()
         model = prob.model
