@@ -357,6 +357,11 @@ class AssembledJacobian(Jacobian):
         if ext_mtx is not None:
             ext_mtx._post_update()
 
+        if self._under_complex_step:
+            # If we create a new _int_mtx while under complex step, we need to convert it to a
+            # complex data type.
+            self._int_mtx.set_complex_step_mode(True)
+
     def _apply(self, system, d_inputs, d_outputs, d_residuals, mode):
         """
         Compute matrix-vector product.
@@ -389,18 +394,19 @@ class AssembledJacobian(Jacobian):
                     mask = ext_mtx._create_mask_cache(d_inputs)
                     self._mask_caches[(d_inputs._names, mode)] = mask
 
+            dresids = d_residuals.asarray()
+
             if mode == 'fwd':
                 if d_outputs._names:
-                    d_residuals._data += int_mtx._prod(d_outputs._data, mode)
+                    dresids += int_mtx._prod(d_outputs.asarray(), mode)
                 if do_mask:
-                    d_residuals._data += ext_mtx._prod(d_inputs._data, mode, mask=mask)
+                    dresids += ext_mtx._prod(d_inputs.asarray(), mode, mask=mask)
 
             else:  # rev
-                dresids = d_residuals._data
                 if d_outputs._names:
-                    d_outputs._data += int_mtx._prod(dresids, mode)
+                    d_outputs += int_mtx._prod(dresids, mode)
                 if do_mask:
-                    d_inputs._data += ext_mtx._prod(dresids, mode, mask=mask)
+                    d_inputs += ext_mtx._prod(dresids, mode, mask=mask)
 
     def set_complex_step_mode(self, active):
         """
