@@ -29,6 +29,8 @@ from openmdao.solvers.solver import Solver
 """
 SQL case database version history.
 ----------------------------------
+12-- OpenMDAO 3.6.1
+     Change key for system metadata to use non-ambiguous separator
 11-- OpenMDAO 3.2
      IndepVarComps are created automatically, so this changes some bookkeeping.
 10-- OpenMDAO 3.0
@@ -54,7 +56,10 @@ SQL case database version history.
 1 -- Through OpenMDAO 2.3
      Original implementation.
 """
-format_version = 11
+format_version = 12
+
+# separator, cannot be a legal char for names
+META_KEY_SEP = '!'
 
 
 def array_to_blob(array):
@@ -398,13 +403,13 @@ class SqliteRecorder(CaseRecorder):
                           "abs2prom=?, prom2abs=?, abs2meta=?, var_settings=?, conns=?",
                           (abs2prom, prom2abs, abs2meta, var_settings_json, conns))
 
-    def record_iteration_driver(self, recording_requester, data, metadata):
+    def record_iteration_driver(self, driver, data, metadata):
         """
         Record data and metadata from a Driver.
 
         Parameters
         ----------
-        recording_requester : object
+        driver : Driver
             Driver in need of recording.
         data : dict
             Dictionary containing desvars, objectives, constraints, responses, and System vars.
@@ -438,15 +443,15 @@ class SqliteRecorder(CaseRecorder):
                            inputs_text, outputs_text, residuals_text))
 
                 c.execute("INSERT INTO global_iterations(record_type, rowid, source) VALUES(?,?,?)",
-                          ('driver', c.lastrowid, recording_requester._get_name()))
+                          ('driver', c.lastrowid, driver._get_name()))
 
-    def record_iteration_problem(self, recording_requester, data, metadata):
+    def record_iteration_problem(self, problem, data, metadata):
         """
         Record data and metadata from a Problem.
 
         Parameters
         ----------
-        recording_requester : object
+        problem : Problem
             Problem in need of recording.
         data : dict
             Dictionary containing desvars, objectives, and constraints.
@@ -458,9 +463,9 @@ class SqliteRecorder(CaseRecorder):
             inputs = data['input']
             residuals = data['residual']
 
-            driver = recording_requester.driver
-            if recording_requester.recording_options['record_derivatives'] and \
-                    driver._designvars and driver._responses:
+            driver = problem.driver
+            if problem.recording_options['record_derivatives'] and \
+               driver._designvars and driver._responses:
                 totals = data['totals']
             else:
                 totals = OrderedDict([])
@@ -496,13 +501,13 @@ class SqliteRecorder(CaseRecorder):
                 c.execute("INSERT INTO global_iterations(record_type, rowid, source) VALUES(?,?,?)",
                           ('problem', c.lastrowid, metadata['name']))
 
-    def record_iteration_system(self, recording_requester, data, metadata):
+    def record_iteration_system(self, system, data, metadata):
         """
         Record data and metadata from a System.
 
         Parameters
         ----------
-        recording_requester : System
+        system : System
             System in need of recording.
         data : dict
             Dictionary containing inputs, outputs, and residuals.
@@ -534,20 +539,20 @@ class SqliteRecorder(CaseRecorder):
                            inputs_text, outputs_text, residuals_text))
 
                 # get the pathname of the source system
-                source_system = recording_requester.pathname
+                source_system = system.pathname
                 if source_system == '':
                     source_system = 'root'
 
                 c.execute("INSERT INTO global_iterations(record_type, rowid, source) VALUES(?,?,?)",
                           ('system', c.lastrowid, source_system))
 
-    def record_iteration_solver(self, recording_requester, data, metadata):
+    def record_iteration_solver(self, solver, data, metadata):
         """
         Record data and metadata from a Solver.
 
         Parameters
         ----------
-        recording_requester : Solver
+        solver : Solver
             Solver in need of recording.
         data : dict
             Dictionary containing outputs, residuals, and errors.
@@ -584,19 +589,19 @@ class SqliteRecorder(CaseRecorder):
                            abs, rel, inputs_text, outputs_text, residuals_text))
 
                 # get the pathname of the source system
-                source_system = recording_requester._system().pathname
+                source_system = solver._system().pathname
                 if source_system == '':
                     source_system = 'root'
 
                 # get solver type from SOLVER class attribute to determine the solver pathname
-                solver_type = recording_requester.SOLVER[0:2]
+                solver_type = solver.SOLVER[0:2]
                 if solver_type == 'NL':
                     source_solver = source_system + '.nonlinear_solver'
                 elif solver_type == 'LS':
                     source_solver = source_system + '.nonlinear_solver.linesearch'
                 else:
                     raise RuntimeError("Solver type '%s' not recognized during recording. "
-                                       "Expecting NL or LS" % recording_requester.SOLVER)
+                                       "Expecting NL or LS" % solver.SOLVER)
 
                 c.execute("INSERT INTO global_iterations(record_type, rowid, source) VALUES(?,?,?)",
                           ('solver', c.lastrowid, source_solver))
@@ -623,20 +628,21 @@ class SqliteRecorder(CaseRecorder):
             except sqlite3.IntegrityError:
                 print("Model viewer data has already has already been recorded for %s." % key)
 
-    def record_metadata_system(self, recording_requester, run_counter=None):
+    def record_metadata_system(self, system, run_number=None):
         """
         Record system metadata.
 
         Parameters
         ----------
-        recording_requester : System
-            The System that would like to record its metadata.
-        run_counter : int or None
-            The number of times run_driver or run_model has been called.
+        system : System
+            The System for which to record metadata.
+        run_number : int or None
+            Number indicating which run the metadata is associated with.
+            None for the first run, 1 for the second, etc.
         """
         if self.connection:
 
-            scaling_vecs, user_options = self._get_metadata_system(recording_requester)
+            scaling_vecs, user_options = self._get_metadata_system(system)
 
             if scaling_vecs is None:
                 return
@@ -654,50 +660,55 @@ class SqliteRecorder(CaseRecorder):
                     pickled_metadata = pickle.dumps(OptionsDictionary(), self._pickle_version)
                     simple_warning("Trying to record option '%s' which cannot be pickled on system "
                                    "%s. Set 'recordable' to False. Skipping recording options for "
-                                   "this system." % (key, recording_requester.msginfo))
+                                   "this system." % (key, system.msginfo))
 
-            path = recording_requester.pathname
+            path = system.pathname
             if not path:
                 path = 'root'
 
             scaling_factors = sqlite3.Binary(scaling_factors)
             pickled_metadata = sqlite3.Binary(pickled_metadata)
 
-            # Need to use OR IGNORE in here because if the user does run_driver more than once
-            #   the current OpenMDAO code will call this function each time and there will be
-            #   SQL errors for "UNIQUE constraint failed: system_metadata.id"
-            # Future versions of OpenMDAO will handle this better.
-            if run_counter is None:
+            if run_number is None:
                 name = path
             else:
-                name = "{}_{}".format(path, str(run_counter))
+                name = META_KEY_SEP.join([path, str(run_number)])
+
             with self.connection as c:
-                c.execute("INSERT OR IGNORE INTO system_metadata"
+                c.execute("INSERT INTO system_metadata"
                           "(id, scaling_factors, component_metadata) "
                           "VALUES(?,?,?)", (name, scaling_factors,
                                             pickled_metadata))
 
-    def record_metadata_solver(self, recording_requester):
+    def record_metadata_solver(self, solver, run_number=None):
         """
         Record solver metadata.
 
         Parameters
         ----------
-        recording_requester : Solver
-            The Solver that would like to record its metadata.
+        solver : Solver
+            The Solver for which to record metadata.
+        run_number : int or None
+            Number indicating which run the metadata is associated with.
+            None for the first run, 1 for the second, etc.
         """
         if self.connection:
-            path = recording_requester._system().pathname
-            solver_class = type(recording_requester).__name__
+            path = solver._system().pathname
+            solver_class = type(solver).__name__
+
             if not path:
                 path = 'root'
+
             id = "{}.{}".format(path, solver_class)
 
-            solver_options = pickle.dumps(recording_requester.options, self._pickle_version)
+            if run_number is not None:
+                id = META_KEY_SEP.join([id, str(run_number)])
+
+            solver_options = pickle.dumps(solver.options, self._pickle_version)
 
             with self.connection as c:
-                c.execute("INSERT INTO solver_metadata(id, solver_options, solver_class) "
-                          "VALUES(?,?,?)", (id, sqlite3.Binary(solver_options), solver_class))
+                c.execute("INSERT INTO solver_metadata(id, solver_options, solver_class)"
+                          " VALUES(?,?,?)", (id, sqlite3.Binary(solver_options), solver_class))
 
     def record_derivatives_driver(self, recording_requester, data, metadata):
         """
