@@ -35,22 +35,6 @@ class NonlinearBlockJac(NonlinearSolver):
 
         self._solver_info.pop()
 
-    def _mpi_print_header(self):
-        """
-        Print header text before solving.
-        """
-        if (self.options['iprint'] > 0):
-
-            pathname = self._system().pathname
-            if pathname:
-                nchar = len(pathname)
-                prefix = self._solver_info.prefix
-                header = prefix + "\n"
-                header += prefix + nchar * "=" + "\n"
-                header += prefix + pathname + "\n"
-                header += prefix + nchar * "="
-                print(header)
-
     def _run_apply(self):
         """
         Run the apply_nonlinear method on the system.
