@@ -185,9 +185,9 @@ class MultiFiCoKriging(object):
     ----------
     corr : Object
         Correlation function to use, default is squared_exponential_correlation.
-    n_features : ndarry
+    n_features : ndarray
         Number of features for each fidelity level.
-    n_samples : ndarry
+    n_samples : ndarray
         Number of samples for each fidelity level.
     nlevel : int
         Number of fidelity levels.
