@@ -32,7 +32,7 @@ class ParametericTestGroup(Group):
             'jacobian_type': ['matvec', 'dense', 'sparse-csc'],
         }
 
-        super(ParametericTestGroup, self).__init__()
+        super().__init__()
 
         self.options.declare('local_vector_class', default='default',
                              values=['default', 'petsc'],
