@@ -495,6 +495,8 @@ def openmdao_cmd():
         user_args = []
 
     parser = argparse.ArgumentParser(description='OpenMDAO Command Line Tools',
+                                     usage='openmdao [-h] [--version] command [command_options] '
+                                     'filename',
                                      epilog='Use -h after any sub-command for sub-command help.'
                                      ' If using a tool on a script that takes its own command line'
                                      ' arguments, place those arguments after a "--". For example:'
@@ -538,7 +540,8 @@ def openmdao_cmd():
 
     # handle case where someone just runs `openmdao <script> [dashed-args]`
     args = [a for a in sys.argv[1:] if not a.startswith('-')]
-    if not set(args).intersection(subs.choices) and len(args) == 1 and os.path.isfile(args[0]):
+    cmdargs = [a for a in sys.argv[1:] if a not in ('-h', '--version')]
+    if not set(args).intersection(subs.choices) and len(args) == 1 and os.path.isfile(cmdargs[0]):
         _load_and_exec(args[0], user_args)
     else:
         hooks.use_hooks = True
