@@ -6,6 +6,11 @@ from openmdao.utils.assert_utils import assert_warning, assert_no_warning
 
 from openmdao.core.explicitcomponent import ExplicitComponent
 
+try:
+    import tabulate
+except ImportError:
+    tabulate = None
+
 
 def check_even(name, value):
     if value % 2 != 0:
@@ -17,6 +22,7 @@ class TestOptionsDict(unittest.TestCase):
     def setUp(self):
         self.dict = OptionsDictionary()
 
+    @unittest.skipIf(tabulate is None, reason="package 'tabulate' is not installed")
     def test_reprs(self):
         class MyComp(ExplicitComponent):
             pass
@@ -32,41 +38,72 @@ class TestOptionsDict(unittest.TestCase):
 
         self.assertEqual(repr(self.dict), repr(self.dict._dict))
 
-        self.assertEqual(self.dict.__str__(width=83), '\n'.join([
-            "========= ============ ================= ===================== ====================",
-            "Option    Default      Acceptable Values Acceptable Types      Description         ",
-            "========= ============ ================= ===================== ====================",
-            "comp      MyComp       N/A               ['ExplicitComponent']                     ",
-            "flag      False        [True, False]     ['bool']                                  ",
-            "long_desc **Required** N/A               ['str']               This description is ",
-            "                                                               long and verbose, so",
-            "                                                                it takes up multipl",
-            "                                                               e lines in the optio",
-            "                                                               ns table.",
-            "test      **Required** ['a', 'b']        N/A                   Test integer value  ",
-            "========= ============ ================= ===================== ====================",
+        self.assertEqual(self.dict.__str__(width=89), '\n'.join([
+            "=========  ============  ===================  =====================  ====================",
+            "Option     Default       Acceptable Values    Acceptable Types       Description",
+            "=========  ============  ===================  =====================  ====================",
+            "comp       MyComp        N/A                  ['ExplicitComponent']",
+            "flag       False         [True, False]        ['bool']",
+            "long_desc  **Required**  N/A                  ['str']                This description is ",
+            "                                                                     long and verbose, so",
+            "                                                                      it takes up multipl",
+            "                                                                     e lines in the optio",
+            "                                                                     ns table.",
+            "test       **Required**  ['a', 'b']           N/A                    Test integer value",
+            "=========  ============  ===================  =====================  ====================",
         ]))
 
         # if the table can't be represented in specified width, then we get the full width version
         self.assertEqual(self.dict.__str__(width=40), '\n'.join([
-            "========= ============ ================= ===================== ====================="
-            "==================================================================== ",
-            "Option    Default      Acceptable Values Acceptable Types      Description          "
-            "                                                                     ",
-            "========= ============ ================= ===================== ====================="
-            "==================================================================== ",
-            "comp      MyComp       N/A               ['ExplicitComponent']                      "
-            "                                                                     ",
-            "flag      False        [True, False]     ['bool']                                   "
-            "                                                                     ",
-            "long_desc **Required** N/A               ['str']               This description is l"
-            "ong and verbose, so it takes up multiple lines in the options table. ",
-            "test      **Required** ['a', 'b']        N/A                   Test integer value   "
-            "                                                                     ",
-            "========= ============ ================= ===================== ====================="
-            "==================================================================== ",
+            "=========  ============  ===================  =====================  ====================="
+            "====================================================================",
+            "Option     Default       Acceptable Values    Acceptable Types       Description",
+            "=========  ============  ===================  =====================  ====================="
+            "====================================================================",
+            "comp       MyComp        N/A                  ['ExplicitComponent']",
+            "flag       False         [True, False]        ['bool']",
+            "long_desc  **Required**  N/A                  ['str']                This description is l"
+            "ong and verbose, so it takes up multiple lines in the options table.",
+            "test       **Required**  ['a', 'b']           N/A                    Test integer value",
+            "=========  ============  ===================  =====================  ====================="
+            "====================================================================",
         ]))
 
+    @unittest.skipIf(tabulate is None, reason="package 'tabulate' is not installed")
+    def test_to_table(self):
+        class MyComp(ExplicitComponent):
+            pass
+
+        my_comp = MyComp()
+
+        self.dict.declare('test', values=['a', 'b'], desc='Test integer value')
+        self.dict.declare('flag', default=False, types=bool)
+        self.dict.declare('comp', default=my_comp, types=ExplicitComponent)
+        self.dict.declare('long_desc', types=str,
+                          desc='This description is long and verbose, so it '
+                               'takes up multiple lines in the options table.')
+
+        expected = "| Option    | Default      | Acceptable Values   | Acceptable Types      " \
+                   "| Description                                                            " \
+                   "                   |\n" \
+                   "|-----------|--------------|---------------------|-----------------------|--" \
+                   "----------------------------------------------------------------------------" \
+                   "-------------|\n" \
+                   "| comp      | MyComp       | N/A                 | ['ExplicitComponent'] |   " \
+                   "                                                                             " \
+                   "           |\n" \
+                   "| flag      | False        | [True, False]       | ['bool']              |   " \
+                   "                                                                             " \
+                   "           |\n" \
+                   "| long_desc | **Required** | N/A                 | ['str']               | Th" \
+                   "is description is long and verbose, so it takes up multiple lines in the opti" \
+                   "ons table. |\n" \
+                   "| test      | **Required** | ['a', 'b']          | N/A                   | Te" \
+                   "st integer value                                                             " \
+                   "           |"
+
+        self.assertEqual(self.dict.to_table(fmt='github'), expected)
+
     def test_type_checking(self):
         self.dict.declare('test', types=int, desc='Test integer value')
 
@@ -129,6 +166,17 @@ class TestOptionsDict(unittest.TestCase):
         self.assertEqual(str(context.exception),
                          "'types' and 'values' were both specified for option 'test3'.")
 
+    def test_check_valid_template(self):
+        # test the template 'check_valid' function
+        from openmdao.utils.options_dictionary import check_valid
+        self.dict.declare('test', check_valid=check_valid)
+
+        with self.assertRaises(ValueError) as context:
+            self.dict['test'] = 1
+
+        expected_msg = "Option 'test' with value 1 is not valid."
+        self.assertEqual(expected_msg, str(context.exception))
+
     def test_isvalid(self):
         self.dict.declare('even_test', types=int, check_valid=check_even)
         self.dict['even_test'] = 2
@@ -269,5 +317,6 @@ class TestOptionsDict(unittest.TestCase):
         with assert_no_warning(DeprecationWarning, msg):
             option = self.dict['test2']
 
+
 if __name__ == "__main__":
     unittest.main()
