@@ -26,6 +26,8 @@ class ModelData {
         this._setParentsAndDepth(this.root, null, 1);
         stopTimer('ModelData._setParentsAndDepth');
 
+        this.md5_hash = modelJSON.md5_hash;
+
         if (this.unconnectedInputs > 0)
             console.info("Unconnected nodes: ", this.unconnectedInputs);
 
@@ -45,7 +47,9 @@ class ModelData {
 
     static uncompressModel(b64str) {
         const compressedData = atob(b64str);
-        const jsonStr = window.pako.inflate(compressedData, { to: 'string' });
+        const jsonStr = window.pako.inflate(compressedData, { to: 'string' })
+            .replace(/([^'"])nan([^'"])/ig, '$1"nan"$2'); // Catch any remaining bare NaNs
+            
         /* for ( let pos = 0; pos < jsonStr.length; pos += 100) {
             console.log(pos, jsonStr.substring(pos, pos+99));
         } */
