@@ -134,6 +134,38 @@ class InfoPropOptions extends InfoPropDefault {
     }
 }
 
+/** Display a subsection of expression values in the info panel for ExecComps */
+class InfoPropExpr extends InfoPropDefault {
+    constructor(key, desc) {
+        super(key, desc, false);
+    }
+
+    /**
+     * There may be a list of expressions, so create a subsection in the table for them.
+     * @param {Object} tbody D3 reference to an existing table body.
+     * @param {N2TreeNode} node Reference to the node that may have the property.
+     */
+    addRow(tbody, node) {
+        if (!this.canShow(node)) return;
+
+        const exprArr = node[this.key];
+
+        // Add a subsection header for the option rows to follow
+        tbody.append('tr').append('th')
+            .text(this.desc)
+            .attr('colspan', '2')
+            .attr('class', 'options-header');
+
+        for (const idx in exprArr) {
+            const splitExpr = exprArr[idx].split(/\s*=\s*/);
+
+            // In case the equals sign is missing for some reason:
+            const displayVal = (splitExpr.length > 1)? splitExpr : [ 'Expr ' + idx, splitExpr[0]];
+            InfoPropDefault.addRowWithVal(tbody, displayVal[0], displayVal[1]);
+        }
+    }
+}
+
 /**
  * Handles properties that are arrays.
  * @typedef InfoPropArray
@@ -235,64 +267,51 @@ class InfoPropArray extends InfoPropDefault {
 
 InfoPropArray.floatFormatter = d3.format('g');
 
-/**
- * Manage the windows that display the values of variables
- * @typedef ValueInfoManager
- */
-class ValueInfoManager {
-    /**
-     * Manage the value info windows.
-     */
-    constructor(ui) {
-        this.ui = ui;
-        this.valueInfoWindows = {};
-    }
-
-    add(name, val) {
-        // Check to see if already exists before opening a new one
-        if (!this.valueInfoWindows[name]) {
-            let valueInfoBox = new ValueInfo(name, val, this.ui);
-            this.valueInfoWindows[name] = valueInfoBox;
-        }
-    }
-
-    remove(name) {
-        this.valueInfoWindows[name].clear() // remove the DOM elements
-        delete this.valueInfoWindows[name]; // remove the reference and let GC cleanup
-    }
-};
-
 /**
  * Manage a window for displaying the value of a variable.
  * @typedef ValueInfo
  */
-class ValueInfo {
+class ValueInfo extends N2WindowResizable {
+
+    /**
+     * Add a new value window if it doesn't already exist.
+     * @param {String} name Variable name.
+     * @param {Number} val Variable value.
+     * @param {PersistentNodeInfo} pnInfo The PersistentNodeInfo window to get data from.
+     * @returns {ValueInfo} The newly constructed window.
+     */
+    static add(name, val, pnInfo) {
+        if (!ValueInfo.existingValueWindows[name]) {
+            ValueInfo.existingValueWindows[name] = true;
+            return new ValueInfo(name, val, pnInfo);
+        }
+    }
+
+    /** Remove the name of the window from the list of existing ones. */
+    static del(name) {
+        if (ValueInfo.existingValueWindows[name]) {
+            delete ValueInfo.existingValueWindows[name];
+        }
+    }
+
     /**
      * Build a list of the properties we care about and set up
      * references to the HTML elements.
      * @param {String} name Variable name.
      * @param {Number} val Variable value.
+     * @param {PersistentNodeInfo} pnInfo The PersistentNodeInfo window to get data from.
      */
-    constructor(name, val, ui) {
+    constructor(name, val, pnInfo) {
+        super('valueInfo-' + uuidv4());
         this.name = name;
         this.val = val;
 
-        /* Construct the DOM elements that make up the window */
-        this.container = d3.select('div#node-value-containers div#template').clone(true);
-        this.container.classed('node-value-hidden', false)
-            .attr('id', 'node-value-' + uuidv4());
-        this.header = this.container.select('.node-value-header');
-        this.table = this.container.select('table');
-        this.title = this.container.select('.node-value-title');
+        this.table = this.body.append('table');
+        this.tbody = this.table.append('tbody');
 
-        this.container.select('.close-value-window-button')
-            .on('click', function () { ui.valueInfoManager.remove(name); });
-
-        this.bringToFront();
-
-        this.update();
-        this._setupDrag();
-        this._setupResizerDrag();
+        this.minWidth = 180;
+        this.minHeight = 100;
+        this.theme('value-info').populate(pnInfo);
     }
 
     /**
@@ -317,42 +336,18 @@ class ValueInfo {
     }
 
     /**
-     * Keep increasing z-index of original info panel to keep it on top.
-     * Max z-index is 2147483647. It will be unusual here for it to climb
-     * above 100, and even extreme cases (e.g. a diagram that's been in use
-     * for weeks with lots of info panels pinned) shouldn't get above a few
-     * thousand.
+     * Fill the table with the data from our val array and display in a window.
      */
-    bringToFront() {
-        // Assign ourselves the current highest info panel z-index and
-        // increment that of the original info panel.
-        const mainPanel = d3.select('#node-info-container');
-        const zIndex = parseInt(mainPanel.style('z-index'));
-        mainPanel.style('z-index', zIndex + 1)
-        this.container.style('z-index', zIndex);
-    }
-
-    clear() {
-        this.container.remove();
-    }
-
-    update() {
-        const titleSpan = this.container.select('.node-value-title');
-
-        // Capture the width of the header before the table is created
-        // We use this to limit how small the window can be as the user resizes
-        this.header_width = parseInt(this.header.style('width'));
-
-        // Check to see if the data is a 2d array since the rest of the code assumes that it is an Array
-        // If only 1d, make it a 2d with one row
+    populate(pnInfo) {
+        // Check to see if the data is a 2d array since the rest of the code assumes
+        // that it is an Array. If only 1d, make it a 2d with one row.
         let val = this.val;
         if (!Array.isArray(val[0])) {
             val = [val];
         }
 
         // Make the top row of the table the indices of the sub-arrays
-        const topRow = this.table.select("tbody")
-            .append('tr');
+        const topRow = this.tbody.append('tr');
         topRow.append('th'); // Top left corner spot is empty
         const valIdxArr = Array.from(val[0].keys());
 
@@ -365,7 +360,7 @@ class ValueInfo {
 
         // Construct the table displaying the variable value
         const evenOdd = ['even', 'odd'];
-        const rows = this.table.select("tbody")
+        const rows = this.tbody
             .selectAll('tr.array-row')
             .data(val)
             .enter()
@@ -381,99 +376,34 @@ class ValueInfo {
             .data(function (row) { return row; })
             .enter()
             .append('td')
-            .text(function (d) { return InfoPropArray.floatFormatter(d); })
+            .text(function (d) { return InfoPropArray.floatFormatter(d); });
+
+        const pnInfoPos = pnInfo._getPos();
+        this.sizeToContent(17, 17) // TODO: Properly find size of scrollbar + 2
+            .title(this.name)
+            .move(pnInfoPos.left + 20, pnInfoPos.top + 20)
+            .show();
 
         // Save the width and height of the table when it is fully
         // constructed. This will be used later to limit the resizing
         // of the window. No need to let the user resize to a size
         // larger than full size
-        this.initial_width = parseInt(this.table.style('width'));
-        this.initial_height = parseInt(this.table.style('height'));
-
-        titleSpan.style('max-width', `${this.initial_width - 50}px`);
-        titleSpan.text(this.name);
+        const pos = this._getPos();
+        this.maxWidth = pos.width;
+        this.maxHeight = pos.height;
+        if (this.minHeight > pos.height) this.minHeight = pos.height;
     }
 
-    /** Listen for the event to begin dragging the value window */
-    _setupDrag() {
-        const self = this;
-
-        this.title.on('mousedown', function () {
-            self.title.style('cursor', 'grabbing');
-            const dragDiv = self.container;
-
-            self.bringToFront();
-
-            dragDiv.style('cursor', 'grabbing')
-                // top style needs to be set explicitly before releasing bottom:
-                .style('top', dragDiv.style('top'))
-                .style('bottom', 'initial');
-
-            self._startPos = [d3.event.clientX, d3.event.clientY]
-            self._offset = [d3.event.clientX - parseInt(dragDiv.style('left')),
-            d3.event.clientY - parseInt(dragDiv.style('top'))];
-
-            const w = d3.select(window)
-                .on("mousemove", e => {
-                    dragDiv
-                        .style('top', (d3.event.clientY - self._offset[1]) + 'px')
-                        .style('left', (d3.event.clientX - self._offset[0]) + 'px');
-                })
-                .on("mouseup", e => {
-                    dragDiv.style('cursor', 'grab');
-                    w.on("mousemove", null).on("mouseup", null);
-
-                });
-
-            d3.event.preventDefault();
-        })
-    }
-
-    /** Set up event handlers for grabbing the bottom corner and dragging */
-    _setupResizerDrag() {
-        const handle = this.container.select('.node-value-resizer-handle');
-        const body = d3.select('body');
-        const tableDiv = this.container.select('.node-value-table-div');
-
-        handle.on('mousedown', e => {
-            const startPos = {
-                'x': d3.event.clientX,
-                'y': d3.event.clientY
-            };
-            const startDims = {
-                'width': parseInt(tableDiv.style('width')),
-                'height': parseInt(tableDiv.style('height'))
-            };
-            body.style('cursor', 'nwse-resize')
-                .on('mouseup', e => {
-                    // Get rid of the drag event handlers
-                    body.style('cursor', 'default')
-                        .on('mousemove', null)
-                        .on('mouseup', null);
-                })
-                .on('mousemove', e => {
-                    let newWidth = d3.event.clientX - startPos.x + startDims.width;
-                    let newHeight = d3.event.clientY - startPos.y + startDims.height;
-
-                    // Do not let get it too big so that you get empty space
-                    newWidth = Math.min(newWidth, this.initial_width);
-                    newHeight = Math.min(newHeight, this.initial_height);
-
-                    // Don't let it get too small or things get weird
-                    newWidth = Math.max(newWidth, this.header_width);
-
-                    tableDiv.style('width', newWidth + 'px');
-                    tableDiv.style('height', newHeight + 'px');
-                });
-
-            d3.event.preventDefault();
-        });
-
+    /** Remove our name from the list of existing windows before closing. */
+    close() {
+        ValueInfo.del(this.name);
+        super.close();
     }
 }
 
 // "Class" variable and function for ValueInfo
 ValueInfo.TRUNCATE_LIMIT = 80;
+ValueInfo.existingValueWindows = {};
 
 /**
  * Manage a table containing all available metadata properties for
@@ -481,12 +411,13 @@ ValueInfo.TRUNCATE_LIMIT = 80;
  * visible or not.
  * @typedef NodeInfo
  */
-class NodeInfo {
+class NodeInfo extends N2Window {
     /**
      * Build a list of the properties we care about and set up
      * references to the HTML elements.
      */
     constructor(ui) {
+        super('nodeInfo-' + uuidv4());
         this.values = {};
 
         // Potential properties
@@ -512,6 +443,8 @@ class NodeInfo {
             new InfoPropYesNo('is_parallel', 'Parallel'),
             new InfoPropDefault('linear_solver', 'Linear Solver'),
             new InfoPropDefault('nonlinear_solver', 'Non-Linear Solver'),
+            new InfoPropExpr('expressions', 'Expressions'),
+
             new InfoPropOptions('options', 'Options'),
             new InfoPropOptions('linear_solver_options', 'Linear Solver Options', 'linear'),
             new InfoPropOptions('nonlinear_solver_options', 'Non-Linear Solver Options', 'nonlinear'),
@@ -525,40 +458,73 @@ class NodeInfo {
         ];
 
         this.ui = ui;
-        this.container = d3.select('#node-info-container');
-        this.table = this.container.select('.node-info-table');
-        this.thead = this.table.select('thead');
-        this.tbody = this.table.select('tbody');
+        this.table = this.body.append('table').attr('class', 'node-info-table');
+        this.tbody = this.table.append('tbody');
         this.toolbarButton = d3.select('#info-button');
-        this.dataDiv = this.container.select('div.node-info-data');
-        this.hidden = true;
+        this.dataDiv = this.main.append('div').attr('class', 'node-info-data');
+        this.theme('node-info');
+        this.hideCloseButton();
+
+        // Becomes active when node info mode is selected on toolbar
+        this.active = false;
     }
 
     /** Make the info box visible if it's hidden */
-    show() {
-        this.toolbarButton.classed('active-tab-icon', true);
+    activate() {
+        this.active = true;
         this.hidden = false;
+        this.toolbarButton.classed('active-tab-icon', true);
         d3.select('#all_pt_n2_content_div').classed('node-data-cursor', true);
+        return this;
     }
 
     /** Make the info box hidden if it's visible */
-    hide() {
-        this.toolbarButton.classed('active-tab-icon', false);
+    deactivate() {
+        this.active = false;
         this.hidden = true;
+        this.toolbarButton.classed('active-tab-icon', false);
         d3.select('#all_pt_n2_content_div').classed('node-data-cursor', false);
+        return this;
     }
 
-    /** Toggle the visibility setting */
+    /** Toggle the active mode */
     toggle() {
-        if (this.hidden) this.show();
-        else this.hide();
+        if (this.active) return this.deactivate();
+        
+        return this.activate();
     }
 
     pin() {
         if (this.tbody.html() == '') return; // Was already pinned so is empty
 
         new PersistentNodeInfo(this);
+        this.hidden = true;
         this.clear();
+        return this;
+    }
+
+    moveNearMouse(event, offset = 15) {
+        if (!this.active) return this;
+
+        return super.moveNearMouse(event, offset);
+    }
+
+    /** Wipe the contents of the table body */
+    clear() {
+        if (!this.active) return;
+        // this.hidden = true;
+
+        this.setList({
+            width: null, height: null, left: null, right: null,
+            top: null, bottom: null
+        })
+        this.dataDiv.html('');
+        this.tbody.html('');
+
+        // Don't just replace with {} because some InfoProps rely
+        // on the reference to this.values:
+        // wipeObj(this.values);
+        return this;
     }
 
     /**
@@ -570,12 +536,12 @@ class NodeInfo {
      * @param {Boolean} [isSolver = false] Whether to use solver properties or not.
      */
     update(event, node, color, isSolver = false) {
-        if (this.hidden) return;
+        if (!this.active) return;
 
         this.clear();
 
         this.name = node.absPathName;
-        this.table.select('tfoot th').style('background-color', color);
+        this.ribbonColor(color);
 
         if (DebugFlags.info && node.hasChildren()) {
             InfoPropDefault.addRowWithVal(this.tbody, 'Children', node.children.length);
@@ -590,77 +556,10 @@ class NodeInfo {
             prop.addRow(this.tbody, node);
         }
 
-        const scrollWidth = this.table.node().scrollWidth,
-            scrollHeight = this.table.node().scrollHeight;
-
-        // Solidify the size of the table after populating so that
-        // it can be positioned reliably by move().
-        this.table
-            .style('width', `${scrollWidth}px`)
-            .style('height', `${scrollHeight}px`)
-
-        // Put the name in the title
-        this.table.select('thead th')
-            .style('background-color', color)
-            .select('.node-info-title')
-            .style('max-width', `${scrollWidth - 100}px`)
-            .text(node.name);
-
-        this.move(event);
-        this.container.classed('info-hidden', false).classed('info-visible', true);
-    }
-
-    /** Wipe the contents of the table body */
-    clear() {
-        if (this.hidden) return;
-        this.container
-            .classed('info-visible', false)
-            .classed('info-hidden', true)
-            .style('width', 'auto')
-            .style('height', 'auto');
-
-        this.table
-            .style('width', 'auto')
-            .style('height', 'auto')
-            .select('thead th span').text('');
-
-        this.dataDiv.html('');
-        this.tbody.html('');
-
-        // Don't just replace with {} because some InfoProps rely
-        // on the reference to this.values:
-        wipeObj(this.values);
-    }
-
-    /**
-     * Relocate the table to a position near the mouse
-     * @param {Object} event The triggering event containing the position.
-     */
-    move(event) {
-        if (this.hidden) return;
-        const offset = 30;
-
-        // Mouse is in left half of window, put box to right of mouse
-        if (event.clientX < window.innerWidth / 2) {
-            this.container.style('right', 'auto');
-            this.container.style('left', (event.clientX + offset) + 'px')
-        }
-        // Mouse is in right half of window, put box to left of mouse
-        else {
-            this.container.style('left', 'auto');
-            this.container.style('right', (window.innerWidth - event.clientX + offset) + 'px');
-        }
-
-        // Mouse is in top half of window, put box below mouse
-        if (event.clientY < window.innerHeight / 2) {
-            this.container.style('bottom', 'auto');
-            this.container.style('top', (event.clientY - offset) + 'px');
-        }
-        // Mouse is in bottom half of window, put box above mouse
-        else {
-            this.container.style('top', 'auto');
-            this.container.style('bottom', (window.innerHeight - event.clientY - offset) + 'px');
-        }
+        this.sizeToContent()
+            .title(node.name)
+            .moveNearMouse(event)
+            .show();
     }
 }
 
@@ -668,45 +567,18 @@ class NodeInfo {
  * Make a persistent copy of the NodeInfo panel and handle its drag/close events
  * @typedef PersistentNodeInfo
  */
-class PersistentNodeInfo {
+class PersistentNodeInfo extends N2WindowDraggable {
     constructor(nodeInfo) {
-        this.orig = nodeInfo.container;
+        super('persistentNodeInfo-' + uuidv4(), '#' + nodeInfo.window.attr('id'));
+
         // Avoid just copying the reference because nodeInfo.values will be wiped:
         this.values = JSON.parse(JSON.stringify(nodeInfo.values));
         this.ui = nodeInfo.ui;
-        this.container = this.orig.clone(true);
-        this.container.classed('persistent-panel', true);
-        this.container.attr('id', uuidv4());
 
-        this.bringToFront();
-
-        this.thead = this.container.select('thead');
-        this.translate = [0, 0];
-
-        const self = this;
-        this.pinButton = this.container.select('#node-info-pin')
-            .on('click', e => { self.unpin(); })
-        this.pinButton.attr('class', 'info-visible');
-
-        this._setupShowMoreButtons(nodeInfo.name);
-        this._setupCopyButtons();
-        this._setupDrag();
-    }
-
-    /**
-     * Keep increasing z-index of original info panel to keep it on top.
-     * Max z-index is 2147483647. It will be unusual here for it to climb
-     * above 100, and even extreme cases (e.g. a diagram that's been in use
-     * for weeks with lots of info panels pinned) shouldn't get above a few
-     * thousand.
-     */
-    bringToFront() {
-        // Assign ourselves the current highest info panel z-index and
-        // increment that of the original info panel.
-        const mainPanel = d3.select('#node-info-container');
-        const zIndex = parseInt(mainPanel.style('z-index'));
-        mainPanel.style('z-index', zIndex + 1)
-        this.container.style('z-index', zIndex);
+        this._setupShowMoreButtons(nodeInfo.name)
+            ._setupCopyButtons()
+            .showCloseButton()
+            .show();
     }
 
     /** Set up event handlers for any "Show More" buttons in the panel */
@@ -715,67 +587,30 @@ class PersistentNodeInfo {
 
         for (const valName in this.values) {
             if (this.values[valName].isTruncated) {
-                this.container.select(`button#${valName}.show_value_button`).on('click', c => {
-                    self.ui.valueInfoManager.add(name, this.values[valName].val);
-                })
+                this.window
+                    .select(`button#${valName}.show_value_button`)
+                    .on('click', c => {
+                        ValueInfo.add(name, self.values[valName].val, self);
+                    })
             }
         }
+
+        return this;
     }
 
     /** Set up event handlers for any "Copy" buttons in the panel */
     _setupCopyButtons() {
-        const self = this;
-
         for (const valName in this.values) {
-            this.container.select(`button#${valName}.copy_value_button`).on('click', c => {
-                const copyText = d3.select("#input-for-pastebuffer");
-                copyText.text(this.values[valName].copyStr);
-                copyText.node().select();
-                document.execCommand('copy');
-            })
+            this.window
+                .select(`button#${valName}.copy_value_button`)
+                .on('click', c => {
+                    const copyText = d3.select("#input-for-pastebuffer");
+                    copyText.text(this.values[valName].copyStr);
+                    copyText.node().select();
+                    document.execCommand('copy');
+                })
         }
 
-    }
-
-    /** Listen for the event to begin dragging a persistent info panel */
-    _setupDrag() {
-        const self = this;
-
-        this.thead.on('mousedown', function () {
-            const dragDiv = self.container;
-
-            self.bringToFront();
-            dragDiv.style('cursor', 'grabbing')
-                .select('th').style('cursor', 'grabbing');
-
-            const dragStart = [d3.event.pageX, d3.event.pageY];
-            let newTrans = [...self.translate];
-
-            const w = d3.select(window)
-                .on("mousemove", e => {
-                    newTrans = [
-                        self.translate[0] + d3.event.pageX - dragStart[0],
-                        self.translate[1] + d3.event.pageY - dragStart[1]
-                    ];
-
-                    dragDiv.style('transform', `translate(${newTrans[0]}px, ${newTrans[1]}px)`)
-                })
-                .on("mouseup", e => {
-                    self.translate = [...newTrans];
-
-                    dragDiv.style('cursor', 'text')
-                        .select('th').style('cursor', 'grab');
-                    w.on("mousemove", null).on("mouseup", null);
-                });
-
-            d3.event.preventDefault();
-        });
-    }
-
-    /** Destroy this instance of the persistent info panel */
-    unpin() {
-        this.thead.on('mousedown', null);
-        this.pinButton.on('click', null);
-        this.container.remove();
+        return this;
     }
 }
